"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generateAppUrl = generateAppUrl;
exports.getEnvOrDefault = exports.getEnv = exports.generateCookieDomain = void 0;
exports.normalizeStringPasswordToMap = normalizeStringPasswordToMap;
var _errors = require("../utils/errors");
var _constants = require("./constants");
/**
 * Return environment variable's value.
 * @throws {@link FronteggEnvNotFound} if not exists
 * @param name
 */
const getEnv = name => {
  const value = process.env[name];
  if (!value) {
    throw new _errors.FronteggEnvNotFound(name);
  }
  return value;
};

/**
 * Return environment variable's value with default if not exists
 * @param {string} name - the name from environment variable {@link EnvVariables}
 * @param {optional string} defaultValue - default value if not exists
 */
exports.getEnv = getEnv;
const getEnvOrDefault = (name, defaultValue) => {
  try {
    return getEnv(name);
  } catch (e) {
    return defaultValue;
  }
};

/**
 * extract hostname from appUrl for session cookie creation
 * @param {string} appUrl - The generated appUrl
 */
exports.getEnvOrDefault = getEnvOrDefault;
const generateCookieDomain = appUrl => {
  try {
    const url = new URL(appUrl);
    return url.hostname.replace(/:(\d)+$/, '');
  } catch (e) {
    throw new _errors.InvalidFronteggEnv(_constants.EnvVariables.FRONTEGG_APP_URL, 'Valid URL');
  }
};

/**
 * Create passwordMap if from {@link EnvVariables.FRONTEGG_ENCRYPTION_PASSWORD}
 * @param {string} password - encryption password
 */
exports.generateCookieDomain = generateCookieDomain;
function normalizeStringPasswordToMap(password) {
  return typeof password === 'string' ? {
    1: password
  } : password;
}

/**
 * Check if the Next.js application running on Vercel Deployment Service
 * and calculate the appUrl based on Vercel Environment variables.
 */
function generateAppUrl() {
  const appUrlEnv = getEnvOrDefault(_constants.EnvVariables.FRONTEGG_APP_URL, undefined);
  const isVercel = getEnvOrDefault(_constants.EnvVariables.VERCEL, undefined) != undefined;
  const vercelUrl = getEnvOrDefault(_constants.EnvVariables.VERCEL_URL, undefined);
  let appUrl;
  if (appUrlEnv) {
    appUrl = appUrlEnv;
  } else if (isVercel && vercelUrl) {
    appUrl = vercelUrl;
  }
  if (!appUrl) {
    if (isVercel) {
      throw new _errors.FronteggEnvNotFound(_constants.EnvVariables.VERCEL, _constants.EnvVariables.VERCEL_URL);
    } else {
      throw new _errors.FronteggEnvNotFound(_constants.EnvVariables.FRONTEGG_APP_URL);
    }
  }
  if (global.customLoginAppUrl) {
    appUrl = global.customLoginAppUrl;
  }

  /**
   * In some cases the {@link EnvVariables.VERCEL_URL} value does not
   * include the URL protocol, bellow code to verify that the appUrl
   * must have a valid http protocol
   */
  if (!appUrl.startsWith('http')) {
    // noinspection HttpUrlsUsage
    const protocol = appUrl.startsWith('localhost') ? 'http://' : 'https://';
    appUrl = `${protocol}${appUrl}`;
  }
  return appUrl;
}
//# sourceMappingURL=helpers.js.map