"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isHostedLoginCallback = exports.handleSessionOnEdge = exports.handleHostedLoginCallback = exports.getSessionOnEdge = exports.checkSessionOnEdge = void 0;
var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));
var _cookies = _interopRequireDefault(require("../utils/cookies"));
var _createSession = _interopRequireDefault(require("../utils/createSession"));
var _encryptionEdge = _interopRequireDefault(require("../utils/encryption-edge"));
var _api = _interopRequireDefault(require("../api"));
var _server = require("next/server");
var _config = _interopRequireDefault(require("../config"));
var _jwt = _interopRequireDefault(require("../utils/jwt"));
var _utils = require("../api/utils");
var _fronteggLogger = _interopRequireDefault(require("../utils/fronteggLogger"));
var _refreshAccessTokenIfNeededOnEdge = require("./refreshAccessTokenIfNeededOnEdge");
var _redirectToLogin = require("./redirectToLogin");
var _shouldBypassMiddleware = require("./shouldBypassMiddleware");
const logger = _fronteggLogger.default.child({
  tag: 'EdgeRuntime.getSessionOnEdge'
});
const handleSessionOnEdge = async params => {
  const {
    request,
    pathname,
    searchParams,
    headers
  } = params;
  if (isHostedLoginCallback(pathname, searchParams)) {
    return handleHostedLoginCallback(request, pathname, searchParams);
  }
  if ((0, _shouldBypassMiddleware.shouldByPassMiddleware)(pathname, headers /*, options: optional bypass configuration */)) {
    return _server.NextResponse.next();
  }
  const edgeSession = await checkSessionOnEdge(request);
  if (!edgeSession) {
    return (0, _redirectToLogin.redirectToLogin)(pathname, searchParams);
  }
  if (edgeSession.headers) {
    return _server.NextResponse.next({
      headers: edgeSession.headers,
      request: {
        headers: edgeSession.forwardedHeaders
      }
    });
  }
  return _server.NextResponse.next();
};
exports.handleSessionOnEdge = handleSessionOnEdge;
const GET_SESSION_ON_EDGE_DEPRECATED_WARN = `Deprecation Notice: getSessionOnEdge has been deprecated. Please use handleSessionOnEdge instead. For example:

file: middleware.ts
\`\`\`ts
  import { NextRequest } from 'next/server';
  import { handleSessionOnEdge } from '@frontegg/nextjs/edge';
    
  export const middleware = async (request: NextRequest) => {
    const { pathname, searchParams } = request.nextUrl;
    const headers = request.headers;
    
    // Additional logic if needed
    
    return handleSessionOnEdge({ request, pathname, searchParams, headers });
  };
  
  
  export const config = {
    matcher: '/(.*)',
  };

\`\`\`

Alternatively, to manually verify the session, you can use checkSessionOnEdge. Note that this method does not redirect to the login page if the session is invalid.
`;

/**
 * getSessionOnEdge is deprecated, please use handleSessionOnEdge instead example:
 *
 * ```ts
 *   import { NextRequest } from 'next/server';
 *   import { handleSessionOnEdge } from '@frontegg/nextjs/edge';
 *
 *   export const middleware = async (request: NextRequest) => {
 *     const { pathname, searchParams } = request.nextUrl;
 *     const headers = request.headers;
 *
 *     // Additional logic if needed
 *
 *     return handleSessionOnEdge({ request, pathname, searchParams, headers });
 *   };
 *
 *   export const config = {
 *     matcher: '/(.*)',
 *   };
 * ```
 * @deprecated
 */

const getSessionOnEdge = (req, disableWarning = false) => {
  const logger = _fronteggLogger.default.child({
    tag: 'EdgeRuntime.getSessionOnEdge'
  });
  const cookies = _cookies.default.getSessionCookieFromRequest(req);
  if (!disableWarning) {
    logger.info(GET_SESSION_ON_EDGE_DEPRECATED_WARN);
  }
  return (0, _createSession.default)(cookies, _encryptionEdge.default);
};

/**
 * Check session on edge and return session if exists this method does not redirect to login page
 * Example:
 *
 * ```ts
 *   import { NextRequest } from 'next/server';
 *   import { handleSessionOnEdge } from '@frontegg/nextjs/edge';
 *
 *   export const middleware = async (request: NextRequest) => {
 *     const { pathname, searchParams } = request.nextUrl;
 *     const headers = request.headers;
 *
 *     // Additional logic if needed
 *
 *     // check if it's a hosted login callback
 *     if (isHostedLoginCallback(pathname, searchParams)) {
 *       return handleHostedLoginCallback(request, pathname, searchParams);
 *     }
 *
 *     // check if we should bypass the middleware
 *     if (shouldByPassMiddleware(pathname)) {
 *       return NextResponse.next();
 *     }
 *
 *     // check session
 *     const session = await checkSessionOnEdge(request);
 *
 *     if (!session) {
 *       return redirectToLogin(pathname);
 *     }
 *
 *     // if headers are present forward them to the next response / request
 *     if (session.headers) {
 *        return NextResponse.next({
 *          headers: edgeSession.headers,
 *          request:{
 *            headers: edgeSession.forwardedHeaders
 *          }
 *        });
 *     }
 *     return NextResponse.next();
 *   };
 * ```
 *
 *
 * @param req
 */
exports.getSessionOnEdge = getSessionOnEdge;
const checkSessionOnEdge = async req => {
  const sessionCookies = _cookies.default.getSessionCookieFromRequest(req);
  let existingSession = await (0, _createSession.default)(sessionCookies, _encryptionEdge.default);
  if (existingSession) {
    logger.debug('session resolved from session cookie');
    return {
      session: existingSession
    };
  }
  logger.debug('Failed to resolve session from cookie, going to refresh token');
  return (0, _refreshAccessTokenIfNeededOnEdge.refreshAccessTokenIfNeededOnEdge)(req);
};
exports.checkSessionOnEdge = checkSessionOnEdge;
async function createSessionFromAccessTokenEdge(data) {
  var _data$accessToken, _data$refreshToken;
  const accessToken = (_data$accessToken = data.accessToken) != null ? _data$accessToken : data.access_token;
  const refreshToken = (_data$refreshToken = data.refreshToken) != null ? _data$refreshToken : data.refresh_token;
  const {
    payload: decodedJwt
  } = await _jwt.default.verify(accessToken);
  decodedJwt.expiresIn = Math.floor((decodedJwt.exp * 1000 - Date.now()) / 1000);
  const tokens = {
    accessToken,
    refreshToken
  };
  const session = await _encryptionEdge.default.sealTokens(tokens, decodedJwt.exp);
  return [session, decodedJwt, refreshToken];
}
const handleHostedLoginCallback = async (req, pathname, searchParams) => {
  var _searchParams$get, _req$headers;
  if (!isHostedLoginCallback(pathname, searchParams)) {
    return _server.NextResponse.next();
  }
  const code = (_searchParams$get = searchParams.get('code')) != null ? _searchParams$get : '';
  let headers = {};
  let clientIp = undefined;
  if (typeof ((_req$headers = req.headers) == null ? void 0 : _req$headers.get) === 'function') {
    var _socket;
    clientIp = (0, _utils.getClientIp)(req.headers.get('cf-connecting-ip') || req.headers.get('x-forwarded-for')) || ((_socket = req.socket) == null ? void 0 : _socket.remoteAddress);
  } else if (typeof req.headers === 'object') {
    var _socket2;
    let requestHeaders = (0, _extends2.default)({}, req.headers);
    clientIp = (0, _utils.getClientIp)(requestHeaders['cf-connecting-ip'] || requestHeaders['x-forwarded-for']) || ((_socket2 = req.socket) == null ? void 0 : _socket2.remoteAddress);
  }
  if (clientIp && _config.default.shouldForwardIp) {
    var _config$sharedSecret;
    headers[_utils.FRONTEGG_FORWARD_IP_HEADER] = clientIp;
    headers[_utils.FRONTEGG_HEADERS_VERIFIER_HEADER] = (_config$sharedSecret = _config.default.sharedSecret) != null ? _config$sharedSecret : '';
    headers[_utils.FRONTEGG_VENDOR_ID_HEADER] = _config.default.clientId;
  }
  const response = await _api.default.exchangeHostedLoginToken((0, _utils.buildRequestHeaders)(headers), code, _config.default.clientId, _config.default.clientSecret);
  const data = await response.json();
  const [session, decodedJwt, refreshToken] = await createSessionFromAccessTokenEdge(data);
  if (!session) {
    return _server.NextResponse.redirect(_config.default.appUrl);
  }
  const isSecured = _config.default.isSSL;
  const cookieValue = _cookies.default.create({
    value: session,
    expires: new Date(decodedJwt.exp * 1000),
    secure: isSecured
  });
  let cookieName = `fe_refresh_${_config.default.clientId.replace('-', '')}`;
  if (_config.default.rewriteCookieByAppId && _config.default.appId) {
    cookieName = `fe_refresh_${_config.default.appId.replace('-', '')}`;
  }
  const refreshCookie = _cookies.default.create({
    cookieName,
    value: refreshToken != null ? refreshToken : '',
    expires: new Date(decodedJwt.exp * 1000),
    secure: isSecured
  });
  const sessionCookieHeaders = cookieValue.map(cookie => ['set-cookie', cookie]);
  const refreshCookieHeaders = refreshCookie.map(cookie => ['set-cookie', cookie]);
  return _server.NextResponse.redirect(_config.default.appUrl, {
    headers: [...sessionCookieHeaders, ...refreshCookieHeaders]
  });
};
exports.handleHostedLoginCallback = handleHostedLoginCallback;
const isHostedLoginCallback = (pathname, searchParams) => {
  if (_config.default.secureJwtEnabled) {
    if (pathname.startsWith('/oauth/callback')) {
      return searchParams.get('code') != null;
    }
  }
  return false;
};
exports.isHostedLoginCallback = isHostedLoginCallback;
//# sourceMappingURL=getSessionOnEdge.js.map