"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _package = _interopRequireDefault(require("next/package.json"));
var _sdkVersion = _interopRequireDefault(require("../sdkVersion"));
var _config = _interopRequireDefault(require("../config"));
var _cookies = _interopRequireDefault(require("../utils/cookies"));
var _fronteggLogger = _interopRequireDefault(require("../utils/fronteggLogger"));
var _utils = require("../api/utils");
var _constants = require("./constants");
const logger = _fronteggLogger.default.child({
  tag: 'FronteggApiMiddleware.ProxyRequestCallback'
});
/**
 * Proxy request callback fired on before each request to Frontegg services,
 * to transport frontegg cookies.
 *
 * @param {ClientRequest} proxyReq - Proxy request to be sent
 * @param {NextApiRequest} req - Next.js incoming request
 */
const ProxyRequestCallback = (proxyReq, req) => {
  try {
    var _req$headers$xFronte, _req$headers$xFronte2;
    logger.info(`${req.url} | Going to proxy request`);
    logger.debug(`${req.url} | parsing request cookies`);
    const allCookies = _cookies.default.parseCookieHeader(req);
    logger.debug(`${req.url} | found ${allCookies} cookies`);
    const fronteggCookiesNames = Object.keys(allCookies).filter(cookieName => {
      return cookieName.startsWith('fe_') && !cookieName.startsWith(_config.default.cookieName);
    });
    logger.debug(`${req.url} | proxy FronteggCookies (${fronteggCookiesNames.join(', ')})`);
    let modifiedCookies = ``;
    fronteggCookiesNames.forEach(requestCookieName => {
      let cookieName = requestCookieName;
      if (_config.default.rewriteCookieByAppId && _config.default.appId) {
        cookieName = requestCookieName.replace(_config.default.appId, _config.default.clientId).replace(_config.default.appId.replace(/-/g, ''), _config.default.clientId.replace(/-/g, '')).replace(_config.default.appId.replace('-', ''), _config.default.clientId.replace('-', ''));
        logger.debug(`cookieName ${requestCookieName} replaced with appId ${cookieName}`);
      }
      logger.debug(`PROXY_ADDING_COOKIE ${cookieName}, ${allCookies[requestCookieName]}`);
      modifiedCookies += `${cookieName}=${allCookies[requestCookieName]}; `;
    });
    proxyReq.setHeader('cookie', modifiedCookies);
    proxyReq.setHeader('x-frontegg-framework', (_req$headers$xFronte = req.headers['x-frontegg-framework']) != null ? _req$headers$xFronte : `next@${_package.default.version}`);
    proxyReq.setHeader('x-frontegg-sdk', (_req$headers$xFronte2 = req.headers['x-frontegg-sdk']) != null ? _req$headers$xFronte2 : `@frontegg/nextjs@${_sdkVersion.default.version}`);
    proxyReq.setHeader('x-frontegg-middleware', 'true');

    /**
     * Request uncompressed responses to avoid handling compression in middleware.
     * This ensures consistent behavior across different environments and simplifies response processing.
     * @see https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Accept-Encoding
     */
    proxyReq.setHeader('Accept-Encoding', 'identity');
    const clientIp = (0, _utils.getClientIp)(req.headers['cf-connecting-ip'] || req.headers['x-forwarded-for']);
    if (clientIp && _config.default.shouldForwardIp) {
      var _config$sharedSecret;
      proxyReq.setHeader(_utils.FRONTEGG_FORWARD_IP_HEADER, clientIp);
      proxyReq.setHeader(_utils.FRONTEGG_HEADERS_VERIFIER_HEADER, (_config$sharedSecret = _config.default.sharedSecret) != null ? _config$sharedSecret : '');
      proxyReq.setHeader(_utils.FRONTEGG_VENDOR_ID_HEADER, _config.default.clientId);
    }
    _constants.headersToRemove.map(header => proxyReq.removeHeader(header));
    logger.debug(`${req.url} | check if request has body`);
    if (req.method !== 'GET' && req.body) {
      logger.debug(`${req.url} | writing request body to proxyReq`);
      const bodyData = JSON.stringify(req.body);
      // in case if content-type is application/x-www-form-urlencoded -> we need to change to application/json
      proxyReq.setHeader('Content-Type', 'application/json');
      proxyReq.setHeader('Content-Length', Buffer.byteLength(bodyData));
      // stream the content
      proxyReq.write(bodyData);
    }
  } catch (e) {
    logger.error(`${req.url} | Failed to proxy request`, e);
  }
};
var _default = exports.default = ProxyRequestCallback;
//# sourceMappingURL=ProxyRequestCallback.js.map