"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _config = _interopRequireDefault(require("../config"));
var _cookies = _interopRequireDefault(require("../utils/cookies"));
var _common = require("../common");
var _helpers = require("./helpers");
var _fronteggLogger = _interopRequireDefault(require("../utils/fronteggLogger"));
var _helpers2 = require("../utils/refreshAccessTokenIfNeeded/helpers");
const logger = _fronteggLogger.default.child({
  tag: 'FronteggApiMiddleware.ProxyResponseCallback'
});
/**
 * Proxy response callback fired on after each response from Frontegg services,
 * to transport frontegg modify cookies and generating encrypted JWT session cookie.
 *
 * @param {IncomingMessage} proxyRes - Proxy response from Frontegg services
 * @param {NextApiRequest} req - Next.js request sent from client-side
 * @param {NextApiResponse} res - Next.js response to send to client-side
 */
const ProxyResponseCallback = (proxyRes, req, res) => {
  let buffer = new Buffer('');
  let totalLength = 0;
  const isSecured = new URL(_config.default.appUrl).protocol === 'https:';
  proxyRes.on('data', chunk => {
    totalLength += chunk.length;
    buffer = Buffer.concat([buffer, chunk], totalLength);
  });
  proxyRes.on('end', async () => {
    try {
      var _proxyRes$statusCode, _CookieManager$modify;
      const url = req.url;
      const statusCode = (_proxyRes$statusCode = proxyRes.statusCode) != null ? _proxyRes$statusCode : 500;
      const isSuccess = statusCode >= 200 && statusCode < 400;
      let bodyStr = buffer.toString('utf-8');
      const isLogout = (0, _helpers.isFronteggLogoutUrl)(url);
      if (isLogout) {
        _cookies.default.removeCookies({
          isSecured,
          cookieDomain: _config.default.cookieDomain,
          res,
          req
        });
        if ((0, _helpers.isFronteggOauthLogoutUrl)(url) || _config.default.isHostedLogin) {
          const {
            asPath: hostedLogoutUrl
          } = (0, _helpers.getHostedLogoutUrl)(req.headers['referer']);
          res.status(302).end(hostedLogoutUrl);
          return;
        }
        res.status(statusCode).end(bodyStr);
        return;
      }
      const cookies = (_CookieManager$modify = _cookies.default.modifySetCookie(proxyRes.headers['set-cookie'], isSecured)) != null ? _CookieManager$modify : [];
      if (isSuccess) {
        try {
          if (bodyStr && bodyStr.length > 0) {
            const tokens = (0, _helpers.extractAccessToken)(bodyStr);
            if (tokens.accessToken) {
              if (process.env['FRONTEGG_SECURE_JWT_ENABLED'] === 'true') {
                bodyStr = JSON.stringify((0, _helpers.removeJwtSignatureFrom)(JSON.parse(bodyStr)));
              }
              const [session, decodedJwt] = await (0, _common.createSessionFromAccessToken)(tokens);
              if (session) {
                const sessionCookie = _cookies.default.create({
                  value: session,
                  expires: new Date(decodedJwt.exp * 1000),
                  secure: isSecured,
                  req
                });
                cookies.push(...sessionCookie);
              }
            }
          }
        } catch (e) {
          /** ignore api call if:
           * - Does not have accessToken / access_token
           * - Not json response
           */
          if (statusCode === 302 && (0, _helpers2.isSSOPostRequest)(url)) {
            /**
             * Ignore saml/oidc postLogin response with redirect
             */
          } else {
            logger.error('failed to create session', e, {
              url,
              statusCode
            });
          }
        }
        Object.keys(proxyRes.headers).filter(header => header !== 'cookie').forEach(header => {
          res.setHeader(header, `${proxyRes.headers[header]}`);
        });
        res.setHeader('set-cookie', cookies);
        res.setHeader('content-length', Buffer.byteLength(bodyStr));
        res.status(statusCode).end(bodyStr);
      } else {
        if (statusCode >= 400 && statusCode !== 404) {
          logger.error(`Middleware request failed statusCode: ${statusCode} for url: ${url}`);
        }
        Object.keys(proxyRes.headers).filter(header => header !== 'cookie').forEach(header => {
          res.setHeader(header, `${proxyRes.headers[header]}`);
        });
        res.setHeader('set-cookie', cookies);
        res.status(statusCode).end(bodyStr);
      }
    } catch (e) {
      logger.error('proxy failed to send request', e);
      res.status(500).end('Internal Server Error');
    }
  });
};
var _default = exports.default = ProxyResponseCallback;
//# sourceMappingURL=ProxyResponseCallback.js.map