"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.splitValueToChunks = exports.getSetCookieHeader = exports.getRefreshTokenCookieNameVariants = exports.getIndexedCookieName = exports.getCookieHeader = void 0;
var _serializer = _interopRequireDefault(require("./serializer"));
var _constants = require("./constants");
var _common = require("../common");
var _config = _interopRequireDefault(require("../../config"));
/**
 * Return a cookieName with index, used for divided cookies.
 *
 * @param {number} index - The index of the cookie, starts with '1'
 * @param {string} _cookieName - Default is {@link config.cookieName}
 */
const getIndexedCookieName = (index, _cookieName) => {
  const cookieName = _cookieName != null ? _cookieName : _config.default.cookieName;
  return `${cookieName}-${index}`;
};

/**
 * Split cookie value if value length exceeded the MAX Length of a standard HTTP header.
 * Used to split the session cookie.
 *
 * @param {string} cookieName - Cookie name for serializing
 * @param {string} value - cookie value to split if needed
 * @param {CookieSerializeOptions} options - {@link CookieSerializeOptions} for serializing
 */
exports.getIndexedCookieName = getIndexedCookieName;
const splitValueToChunks = (cookieName, value, options) => {
  const cookieOptionLength = _serializer.default.serialize(getIndexedCookieName(1, cookieName), '', options).length;
  const chunkSize = _constants.COOKIE_MAX_LENGTH - cookieOptionLength - 1;
  const valueChunks = (0, _common.chunkString)(value, chunkSize);
  return valueChunks.map((chunk, index) => {
    const indexedCookieName = getIndexedCookieName(index + 1, cookieName);
    return _serializer.default.serialize(indexedCookieName, chunk, options) + ';';
  });
};

/**
 * Receive incoming http request, and extract the cookie header.
 * @return cookie as string if exists, else empty string
 *
 * @param {RequestType} request - Incoming HTTP Request
 */
exports.splitValueToChunks = splitValueToChunks;
const getCookieHeader = request => {
  let cookieHeader;
  if ('credentials' in request) {
    cookieHeader = request.headers.get('cookie');
  } else {
    cookieHeader = request.headers.cookie;
  }
  if (!cookieHeader) {
    return '';
  }
  return cookieHeader;
};

/**
 * Receive http response, and extract the set-cookie header.
 * @return cookie as string if exists, else empty string
 *
 * @param {ResponseType} response - HTTP Response
 */
exports.getCookieHeader = getCookieHeader;
const getSetCookieHeader = response => {
  let cookieHeader = response.getHeader('set-cookie');
  const cookies = [];
  if (!Array.isArray(cookieHeader)) {
    cookies.push(`${cookieHeader}`);
  } else {
    cookies.push(...cookieHeader);
  }
  return cookies.map(cookie => cookie.split(';')[0]).join(';');
};
exports.getSetCookieHeader = getSetCookieHeader;
const getRefreshTokenCookieNameVariants = () => {
  if (_config.default.rewriteCookieByAppId && _config.default.appId) {
    return [`fe_refresh_${_config.default.appId}`, `fe_refresh_${_config.default.appId.replace('-', '')}`, `fe_refresh_${_config.default.appId.replace(/-/g, '')}`];
  } else {
    return [`fe_refresh_${_config.default.clientId}`, `fe_refresh_${_config.default.clientId.replace('-', '')}`, `fe_refresh_${_config.default.clientId.replace(/-/g, '')}`];
  }
};
exports.getRefreshTokenCookieNameVariants = getRefreshTokenCookieNameVariants;
//# sourceMappingURL=helpers.js.map