"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _serializer = _interopRequireDefault(require("./serializer"));
var _config = _interopRequireDefault(require("../../config"));
var _constants = require("./constants");
var _helpers = require("./helpers");
var _fronteggLogger = _interopRequireDefault(require("../fronteggLogger"));
class CookieManager {
  constructor() {
    this.getCookieName = (cookieNumber, cookieName = _config.default.cookieName) => cookieNumber ? (0, _helpers.getIndexedCookieName)(cookieNumber, cookieName) : cookieName;
    this.createEmptySingleCookie = (cookieName, isSecured, cookieDomain) => {
      return this.create({
        cookieName,
        value: '',
        expires: new Date(),
        secure: isSecured,
        domain: cookieDomain,
        silent: true
      });
    };
    this.createEmptyCookies = (isSecured, cookieDomain, _cookiesToRemove, removeRefresh = true) => {
      const allEmptyCookies = [];
      const refreshTokenVariants = (0, _helpers.getRefreshTokenCookieNameVariants)();
      const cookiesToRemove = [..._cookiesToRemove, ...(removeRefresh ? refreshTokenVariants : [])];
      cookiesToRemove.forEach(name => {
        allEmptyCookies.push(...this.createEmptySingleCookie(name, isSecured, cookieDomain));
      });
      return allEmptyCookies;
    };
    this.getCookiesToRemove = request => {
      const logger = _fronteggLogger.default.child({
        tag: 'getCookiesToRemove'
      });
      if (!request) {
        return [];
      }
      try {
        logger.info('extract cookie from request headers');
        const cookieStr = (0, _helpers.getCookieHeader)(request);
        const cookies = cookieStr && _serializer.default.parse(cookieStr);
        if (!cookies) {
          return [];
        }
        let cookieNumber = 1;
        const cookieToRemove = [];
        if (cookies[this.getCookieName()]) {
          cookieToRemove.push(this.getCookieName());
        }
        while (cookies[this.getCookieName(cookieNumber)]) {
          cookieToRemove.push(this.getCookieName(cookieNumber));
          cookieNumber++;
        }
        logger.info(`number of cookies to remove: ${cookieToRemove.length}`);
        return cookieToRemove;
      } catch (e) {
        logger.error(e);
        return [];
      }
    };
    /**
     * Take a list of cookie headers and modify the Domain / Secure / SameSite
     * to proxy the cookies to Frontegg Services and vice-versa
     * @param {string[]} setCookieValue - list of cookies to modify
     * @param {boolean} isSecured - if the running application behind SSL
     */
    this.modifySetCookie = (setCookieValue, isSecured) => {
      const logger = _fronteggLogger.default.child({
        tag: 'CookieManager.modifySetCookie'
      });
      if (!setCookieValue || setCookieValue.length === 0) {
        logger.info(`No headers to modify`);
        return setCookieValue;
      }

      // noinspection SuspiciousTypeOfGuard
      if (typeof setCookieValue === 'string') {
        const cookieRegexSplitter = /(.*?;.*?(?:Expires=[^;]+;.*?)?(?=,\s*\S+=|$))/g;
        const cookies = [];
        const cookieHeader = `${setCookieValue}`;
        let match;
        while ((match = cookieRegexSplitter.exec(cookieHeader)) !== null) {
          let cookieString = match[1].trim();
          if (cookieString.startsWith(',')) {
            cookies.push(cookieString.substring(1).trim());
          } else {
            cookies.push(cookieString);
          }
        }
        setCookieValue = cookies;
      }
      logger.info(`modifying cookie headers (count: ${setCookieValue.length})`);
      return setCookieValue.map(c => {
        let cookie = c.split('; ');
        logger.debug(`modifying cookie ${cookie[0]}, isSecured: ${isSecured}`);
        if (!isSecured) {
          cookie = cookie.filter(property => property !== 'Secure' && property !== 'SameSite=None');
        }
        return cookie.map(property => {
          if (property.trim().startsWith(`fe_refresh_${_config.default.clientId.replace('-', '')}`)) {
            if (_config.default.rewriteCookieByAppId && _config.default.appId) {
              return property.replace(`fe_refresh_${_config.default.clientId.replace('-', '')}`, `fe_refresh_${_config.default.appId.replace('-', '')}`);
            } else {
              return property;
            }
          } else if (property.toLowerCase() === `domain=${_config.default.baseUrlHost}`) {
            return `Domain=${_config.default.cookieDomain}`;
          } else {
            return property;
          }
        }).join(';') + ';';
      });
    };
  }
  get refreshTokenKey() {
    if (_config.default.rewriteCookieByAppId && _config.default.appId) {
      return `fe_refresh_${_config.default.appId.replace(/-/g, '')}`;
    } else {
      return `fe_refresh_${_config.default.clientId.replace(/-/g, '')}`;
    }
  }

  /**
   * This function creates list of empty cookies that already exists in the request.
   * This is used for removing existing cookies before creating new ones.
   * @param {CreateCookieOptions} options - Create cookie options
   */
  getEmptyCookiesBeforeCreatingNew({
    req,
    value,
    secure,
    domain
  }, newCookies) {
    if (!req || !value) {
      return [];
    }
    const newCookieNames = newCookies.map(c => c.split('=')[0]);
    const cookiesToRemove = this.getCookiesToRemove(req).filter(cookie => newCookieNames.indexOf(cookie) === -1);
    if (cookiesToRemove.length === 0) {
      return [];
    }
    return this.createEmptyCookies(secure, domain != null ? domain : _config.default.cookieDomain, cookiesToRemove, false);
  }

  /**
   * Validate and create new cookie headers.
   * The default value of `cookieName` is {@link config.cookieName}
   * @param {CreateCookieOptions} options - Create cookie options
   */
  create(options) {
    var _options$cookieName, _options$httpOnly, _options$domain, _options$path;
    const logger = _fronteggLogger.default.child({
      tag: 'CookieManager.create',
      level: options.silent ? 'error' : undefined
    });
    const cookieName = (_options$cookieName = options.cookieName) != null ? _options$cookieName : this.getCookieName();
    const cookieValue = options.value;
    logger.info(`Creating new cookie for '${cookieName}'`);
    const serializeOptions = {
      expires: options.expires,
      httpOnly: (_options$httpOnly = options.httpOnly) != null ? _options$httpOnly : true,
      domain: (_options$domain = options.domain) != null ? _options$domain : _config.default.cookieDomain,
      path: (_options$path = options.path) != null ? _options$path : '/',
      priority: 'high'
    };
    if (options.secure) {
      logger.debug(`Set cookie '${cookieName}' as secure`);
      serializeOptions.secure = options.secure;
      serializeOptions.sameSite = _config.default.cookieSameSite;
    }
    const serializedCookie = _serializer.default.serialize(cookieName, cookieValue, serializeOptions);
    let newCookies = [];
    if (serializedCookie.length <= _constants.COOKIE_MAX_LENGTH) {
      logger.info(`Successfully create a cookie header, '${cookieName}'`);
      newCookies = [serializedCookie];
    } else {
      logger.debug('Going to split cookie into chunks');
      /** Create chunked cookie headers and store value as array of headers */
      const cookies = (0, _helpers.splitValueToChunks)(cookieName, cookieValue, serializeOptions);
      logger.info(`Successfully create chunked cookie headers, '${cookieName}' (count: ${cookies.length})`);
      newCookies = cookies;
    }
    const removedCookiesValue = this.getEmptyCookiesBeforeCreatingNew(options, newCookies);
    return [...removedCookiesValue, ...newCookies];
  }

  /**
   * Receive incoming http request, and extract the cookie header.
   * @return cookie as string if exists, else empty string
   *
   * @param {RequestType} request - Incoming HTTP Request
   */
  parseCookieHeader(request) {
    const logger = _fronteggLogger.default.child({
      tag: 'CookieManager.parseCookieHeader'
    });
    logger.info('Going to extract all cookies header from request');
    const cookieHeader = (0, _helpers.getCookieHeader)(request);
    logger.info('Parsing cookie header to map');
    return _serializer.default.parse(cookieHeader);
  }

  /**
   * Loop over cookie headers, extract, parse cookies and merged divided cookies from incoming http request,
   * @return full session cookie headers if exists, else return undefined
   * @param {RequestType} request - Incoming HTTP Request
   */
  getSessionCookieFromRequest(request) {
    const logger = _fronteggLogger.default.child({
      tag: 'CookieManager.getSessionCookieFromRequest'
    });
    logger.info('Going to extract session cookies header from request');
    if (!request) {
      logger.info(`'request' argument is null, Cookie header not found`);
      return undefined;
    }
    logger.debug('Getting cookie header');
    const cookieStr = (0, _helpers.getCookieHeader)(request);
    logger.debug('Parsing cookie header string');
    const cookies = _serializer.default.parse(cookieStr);
    logger.debug('Loop over session cookie headers');
    let i = 1;
    let sessionCookies = '';
    let sessionCookieChunk = cookies[this.getCookieName()];
    if (sessionCookieChunk === undefined) {
      do {
        sessionCookieChunk = cookies[(0, _helpers.getIndexedCookieName)(i++)];
        if (sessionCookieChunk) {
          sessionCookies += sessionCookieChunk;
        }
      } while (sessionCookieChunk);
    } else {
      sessionCookies = sessionCookieChunk;
    }
    if (sessionCookies.length === 0) {
      logger.info('Session cookie NOT found');
      return undefined;
    }
    logger.info(`Session cookie found, (count: ${sessionCookies.length})`);
    return sessionCookies;
  }
  getRefreshCookieFromRequestEdge(request) {
    const logger = _fronteggLogger.default.child({
      tag: 'CookieManager.getRefreshCookieFromRequest'
    });
    logger.info('Going to extract refresh cookie header from request');
    if (!request) {
      logger.info(`'request' argument is null, Cookie header not found`);
      return undefined;
    }
    logger.debug('Getting cookie header');
    const cookieStr = (0, _helpers.getCookieHeader)(request);
    logger.debug('Parsing cookie header string');
    const cookies = _serializer.default.parse(cookieStr);
    const refreshTokenKey = this.refreshTokenKey;
    const refreshCookie = Object.keys(cookies).find(cookie => {
      return cookie.replace(/-/g, '') === refreshTokenKey;
    });
    if (!refreshCookie) {
      logger.info('Refresh cookie NOT found');
      return undefined;
    }
    logger.info(`Refresh cookie found for key: ${refreshTokenKey}`);
    return cookies[refreshCookie];
  }

  /**
   * Loop over cookie headers, extract, parse cookies and merged divided cookies from redirected http response,
   * @return full session cookie headers if exists, else return undefined
   * @param {ResponseType} response - Outgoing HTTP response
   */
  getSessionCookieFromRedirectedResponse(response) {
    const logger = _fronteggLogger.default.child({
      tag: 'CookieManager.getSessionCookieFromRedirectedResponse'
    });
    logger.info('Going to extract session from set-cookie header from response');
    if (!response) {
      logger.info(`'response' argument is null, Cookie header not found`);
      return undefined;
    }
    logger.debug('Getting set-cookie header');
    const cookieStr = (0, _helpers.getSetCookieHeader)(response);
    logger.debug('Parsing set-cookie header string');
    const cookies = _serializer.default.parse(cookieStr);
    logger.debug('Loop over session set-cookie header');
    let i = 1;
    let sessionCookies = '';
    let sessionCookieChunk = cookies[this.getCookieName()];
    if (sessionCookieChunk === undefined) {
      do {
        sessionCookieChunk = cookies[(0, _helpers.getIndexedCookieName)(i++)];
        if (sessionCookieChunk) {
          sessionCookies += sessionCookieChunk;
        }
      } while (sessionCookieChunk);
    } else {
      sessionCookies = sessionCookieChunk;
    }
    if (sessionCookies.length === 0) {
      logger.info('Session set-cookie NOT found');
      return undefined;
    }
    logger.info(`Session set-cookie found, (count: ${sessionCookies.length})`);
    return sessionCookies;
  }
  parseCookieFromArray(cookies) {
    const logger = _fronteggLogger.default.child({
      tag: 'CookieManager.parseCookieFromArray'
    });
    const cookieChunks = cookies.filter(c => c.name.includes(this.getCookieName()));
    logger.info('Parsing session cookie from RequestCookie for Next.JS 13+');
    if (!cookieChunks || cookieChunks.length === 0) {
      logger.info(`No session cookies found`);
      return undefined;
    }
    logger.debug(`Found ${cookieChunks.length} chunks`);
    cookieChunks.sort((a, b) => {
      const firstCookieNumber = parseInt(a.name.slice(-1));
      const secondCookieNumber = parseInt(b.name.slice(-1));
      return firstCookieNumber > secondCookieNumber ? 1 : -1;
    });
    logger.info(`Concatenate session cookies chunks`);
    return cookieChunks.map(c => c.value).join('');
  }
  /**
   * Take a list of cookieNames and return the headers to remove the cookies from the client side
   * @param {RemoveCookiesOptions} setCookieValue
   */
  getRequestCookiesHeaderToRemove({
    cookieNames,
    isSecured,
    cookieDomain,
    req
  }) {
    const logger = _fronteggLogger.default.child({
      tag: 'CookieManager.getSetCookiesHeaderToRemove'
    });
    logger.debug('Setting empty cookie headers remove cookies from client side');
    const cookiesToRemove = this.getCookiesToRemove(req);
    const cookieValue = this.createEmptyCookies(isSecured, cookieDomain, cookieNames != null ? cookieNames : cookiesToRemove);
    return [...cookieValue];
  }

  /**
   * Take a list of cookieNames and modify request/response headers
   * to proxy the cookies from Next.js to Frontegg Services and vice-versa
   * @param {RemoveCookiesOptions} setCookieValue
   */
  removeCookies({
    cookieNames,
    isSecured,
    cookieDomain,
    res,
    req
  }) {
    var _ref;
    const logger = _fronteggLogger.default.child({
      tag: 'CookieManager.removeCookies'
    });
    const cookieValue = this.getRequestCookiesHeaderToRemove({
      cookieNames,
      isSecured,
      cookieDomain,
      req
    });
    let existingSetCookie = (_ref = res.getHeader('set-cookie')) != null ? _ref : [];
    if (existingSetCookie != null && typeof existingSetCookie === 'object' && !Array.isArray(existingSetCookie)) {
      existingSetCookie = Object.values(existingSetCookie);
    }
    if (typeof existingSetCookie === 'string') {
      existingSetCookie = [existingSetCookie];
    }
    const setCookieHeaders = [...existingSetCookie, ...cookieValue];
    logger.debug(`removing headers (count: ${setCookieHeaders.length})`);
    res.setHeader('set-cookie', setCookieHeaders);
  }
}
var _default = exports.default = new CookieManager();
//# sourceMappingURL=index.js.map