"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getForwardedSession = getForwardedSession;
exports.hasRefreshTokenCookie = hasRefreshTokenCookie;
exports.hasSetSessionCookie = hasSetSessionCookie;
exports.isOauthCallback = isOauthCallback;
exports.isRuntimeNextRequest = isRuntimeNextRequest;
exports.isSSOPostRequest = isSSOPostRequest;
exports.isSamlCallback = isSamlCallback;
exports.refreshAccessTokenEmbedded = refreshAccessTokenEmbedded;
exports.refreshAccessTokenHostedLogin = refreshAccessTokenHostedLogin;
exports.saveForwardedSession = saveForwardedSession;
var _fronteggLogger = _interopRequireDefault(require("../fronteggLogger"));
var _cookies = _interopRequireDefault(require("../cookies"));
var _api = _interopRequireDefault(require("../../api"));
var _common = require("../../common");
var _config = _interopRequireDefault(require("../../config"));
var _constants = require("../common/constants");
function hasRefreshTokenCookie(cookies) {
  if (cookies == null) {
    return false;
  }
  const logger = _fronteggLogger.default.child({
    tag: 'refreshToken.hasRefreshTokenCookie'
  });
  const refreshTokenKey = _cookies.default.refreshTokenKey;
  logger.debug(`Checking if '${refreshTokenKey}' exists in cookies`);
  const cookieKey = Object.keys(cookies).find(cookie => {
    return cookie.replace(/-/g, '') === refreshTokenKey;
  });
  const exists = cookieKey != null;
  logger.debug(`Cookie '${refreshTokenKey}' ${exists ? 'exists' : 'NOT exists'} in cookies`);
  return exists;
}
async function refreshAccessTokenEmbedded(request) {
  const logger = _fronteggLogger.default.child({
    tag: 'refreshToken.refreshAccessTokenEmbedded'
  });
  const headers = request.headers;
  const cookies = request.cookies;
  logger.info('check if has refresh token headers');
  if (hasRefreshTokenCookie(cookies)) {
    logger.info('going to refresh token (embedded mode)');
    if (_config.default.appId) {
      headers['frontegg-requested-application-id'] = _config.default.appId;
    }
    return await _api.default.refreshTokenEmbedded(headers);
  }
  return null;
}
async function refreshAccessTokenHostedLogin(request) {
  const logger = _fronteggLogger.default.child({
    tag: 'refreshToken.refreshAccessTokenHostedLogin'
  });
  const headers = request.headers;
  logger.info('trying to get token from cookies');
  const sealFromCookies = _cookies.default.getSessionCookieFromRequest(request);
  try {
    const tokens = await (0, _common.getTokensFromCookie)(sealFromCookies);
    if (!(tokens != null && tokens.refreshToken)) {
      logger.info('refresh token not found');
      return null;
    }
    if (_config.default.appId) {
      headers['frontegg-requested-application-id'] = _config.default.appId;
    }
    if (_config.default.secureJwtEnabled) {
      const clientId = _config.default.clientId;
      const clientSecret = _config.default.clientSecret;
      logger.info('going to refresh token (hosted-login mode) (secure-jwt mode)');
      return await _api.default.refreshTokenHostedLogin(headers, tokens.refreshToken, clientId, clientSecret);
    } else {
      logger.info('going to refresh token (hosted-login mode) ', tokens.refreshToken);
      return await _api.default.refreshTokenHostedLogin(headers, tokens.refreshToken);
    }
  } catch (e) {
    logger.error(e);
    return null;
  }
}

/**
 * If url starts with /_next/ means that the user trying to navigate
 * to a new nextjs page, in this scenario no need to refresh token
 * we can just return the actual stateless session from
 * the encrypted cookie
 */
function isRuntimeNextRequest(url) {
  return url.startsWith('/_next/');
}

/**
 * If url starts with '/oauth/callback' means that the user navigated back
 * from frontegg hosted login, in this scenario no need to SSR refresh token
 */
function isOauthCallback(url) {
  return url.startsWith('/oauth/callback');
}

/**
 * If url starts with '/account/saml/callback' means that the user navigated back
 * from sso login, in this scenario no need to SSR refresh token
 */
function isSamlCallback(url) {
  return url.startsWith('/account/saml/callback') || url.startsWith('/account/oidc/callback');
}

/**
 * If the url equals to '/frontegg/auth/{provider}/callback', it means that the SSO provider
 * is posting an http request to the nextjs backend middleware after successfully logged in the user
 */
function isSSOPostRequest(url) {
  return url === '/frontegg/auth/saml/callback' || url === '/frontegg/auth/oidc/callback';
}

/**
 * This function verifies if the headers includes a 'set-cookie' header
 * from a prior refresh token request. If it's the case, we can infer that the
 * session cookie has been initialized, thus we can disable the double refresh token
 * for server-side redirects such as '/_error' or any other server-side redirects.
 */
function hasSetSessionCookie(cookieHeader) {
  if (!cookieHeader || typeof cookieHeader === 'number') {
    return false;
  }
  const cookieName = _config.default.cookieName;
  if (typeof cookieHeader === 'string') {
    return cookieHeader.indexOf(cookieName) !== -1;
  }
  if (Array.isArray(cookieHeader)) {
    return cookieHeader.some(header => header.startsWith(cookieName));
  }
  return false;
}

/**
 * This function stores the Frontegg session instance for use
 * within the Next.js application during the token refresh process
 * in the redirect request page.
 */
function saveForwardedSession(holder, session) {
  holder[_constants.FRONTEGG_FORWARDED_SESSION_KEY] = session;
}

/**
 * This function retrieves the stored session from the previous redirected page request.
 * This helps in preventing the token from being refreshed twice during a single client page request.
 */

function getForwardedSession(holder) {
  return holder[_constants.FRONTEGG_FORWARDED_SESSION_KEY];
}
//# sourceMappingURL=helpers.js.map