"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = refreshAccessTokenIfNeeded;
Object.defineProperty(exports, "isRuntimeNextRequest", {
  enumerable: true,
  get: function () {
    return _helpers.isRuntimeNextRequest;
  }
});
var _common = require("../../common");
var _config = _interopRequireDefault(require("../../config"));
var _cookies = _interopRequireDefault(require("../cookies"));
var _helpers = require("./helpers");
var _fronteggLogger = _interopRequireDefault(require("../fronteggLogger"));
var _encryption = _interopRequireDefault(require("../encryption"));
var _createSession = _interopRequireDefault(require("../createSession"));
var _utils = require("../../api/utils");
/**
 * Refreshes the access token for the current session.
 *
 * @param {NextPageContext} ctx - The Next.js Page Context object.
 * @returns {Promise<FronteggNextJSSession | null>} A Promise that resolves to the updated session object, or `null` if the refresh failed.
 */
async function refreshAccessTokenIfNeeded(ctx) {
  const logger = _fronteggLogger.default.child({
    tag: 'refreshAccessTokenIfNeeded'
  });
  logger.info(`Refreshing token by for PageContext ${ctx.pathname}`);
  const nextJsRequest = ctx.req;
  const nextJsResponse = ctx.res;
  const url = nextJsRequest == null ? void 0 : nextJsRequest.url;
  if (!nextJsResponse || !nextJsRequest || !url) {
    logger.debug(`abandon refreshToken due to PageContext.req = null`);
    return null;
  }
  if ((0, _helpers.hasSetSessionCookie)(nextJsResponse.getHeader('set-cookie'))) {
    const cookies = _cookies.default.getSessionCookieFromRedirectedResponse(nextJsResponse);
    const session = await (0, _createSession.default)(cookies, _encryption.default);
    logger.debug('Abandon refreshToken due to a previous redirect to /_error or other server-side redirect.');
    return session != null ? session : (0, _helpers.getForwardedSession)(nextJsResponse);
  }
  try {
    var _nextJsRequest$socket, _ref, _ref2, _response$headers$raw, _response$headers, _response$headers$raw2, _response$headers2, _response$headers2$ge, _response$headers3, _response$headers3$ge, _CookieManager$modify, _data$accessToken;
    logger.info(`Check if should request made from first application load`);
    if ((0, _helpers.isRuntimeNextRequest)(url) || _config.default.disableInitialPropsRefreshToken) {
      logger.debug(`Detect runtime next.js request, resolving existing session from cookies if exists`);
      const cookies = _cookies.default.getSessionCookieFromRequest(nextJsRequest);
      const session = await (0, _createSession.default)(cookies, _encryption.default);
      if (session) {
        logger.debug(`session resolved from session cookie`);
        return session;
      } else {
        logger.info('Failed to resolve session from cookie, going to refresh token');
      }
    }
    if (_config.default.isHostedLogin) {
      // hosted login bypassed urls
      if ((0, _helpers.isOauthCallback)(url)) {
        logger.debug(`abandon refreshToken for HostedLogin Callback ${url}`);
        _cookies.default.removeCookies({
          isSecured: _config.default.isSSL,
          cookieDomain: _config.default.cookieDomain,
          res: nextJsResponse,
          req: nextJsRequest
        });
      }
    } else {
      // embedded login bypassed urls
      if ((0, _helpers.isSamlCallback)(url)) {
        logger.debug(`abandon refreshToken for Saml Callback ${url}`);
        return null;
      }
    }
    const clientIp = (0, _utils.getClientIp)(nextJsRequest.headers['cf-connecting-ip'] || nextJsRequest.headers['x-forwarded-for']) || ((_nextJsRequest$socket = nextJsRequest.socket) == null ? void 0 : _nextJsRequest$socket.remoteAddress);
    if (clientIp && _config.default.shouldForwardIp) {
      var _config$sharedSecret;
      nextJsRequest.headers[_utils.FRONTEGG_FORWARD_IP_HEADER] = clientIp;
      nextJsRequest.headers[_utils.FRONTEGG_HEADERS_VERIFIER_HEADER] = (_config$sharedSecret = _config.default.sharedSecret) != null ? _config$sharedSecret : '';
      nextJsRequest.headers[_utils.FRONTEGG_VENDOR_ID_HEADER] = _config.default.clientId;
    }
    let response;
    if (_config.default.isHostedLogin) {
      response = await (0, _helpers.refreshAccessTokenHostedLogin)(nextJsRequest);
    } else {
      response = await (0, _helpers.refreshAccessTokenEmbedded)(nextJsRequest);
    }
    const isSecured = _config.default.isSSL;
    if (response === null || !response.ok) {
      _cookies.default.removeCookies({
        cookieDomain: _config.default.cookieDomain,
        isSecured,
        req: nextJsRequest,
        res: nextJsResponse
      });
      return null;
    }
    const data = await response.json();
    const cookieHeader = // @ts-ignore the first argument "raw" will only work before nextjs 13.4 and the second argument "getSetCookie" will only work after
    (_ref = (_ref2 = (_response$headers$raw = (_response$headers = response.headers) == null || (_response$headers$raw2 = _response$headers.raw) == null ? void 0 : _response$headers$raw2.call(_response$headers)['set-cookie']) != null ? _response$headers$raw : // @ts-ignore the first argument "raw" will only work before nextjs 13.4 and the second argument "getSetCookie" will only work after
    (_response$headers2 = response.headers) == null || (_response$headers2$ge = _response$headers2.getSetCookie) == null ? void 0 : _response$headers2$ge.call(_response$headers2)) != null ? _ref2 : (_response$headers3 = response.headers) == null || (_response$headers3$ge = _response$headers3.get) == null ? void 0 : _response$headers3$ge.call(_response$headers3, 'set-cookie')) != null ? _ref : [];
    const newSetCookie = (_CookieManager$modify = _cookies.default.modifySetCookie(cookieHeader, isSecured)) != null ? _CookieManager$modify : [];
    const [session, decodedJwt, refreshToken] = await (0, _common.createSessionFromAccessToken)(data);
    if (!session) {
      return null;
    }
    const cookieValue = _cookies.default.create({
      value: session,
      expires: new Date(decodedJwt.exp * 1000),
      secure: isSecured,
      req: nextJsRequest
    });
    newSetCookie.push(...cookieValue);
    nextJsResponse.setHeader('set-cookie', newSetCookie);
    const fronteggSession = {
      accessToken: (_data$accessToken = data.accessToken) != null ? _data$accessToken : data.access_token,
      user: decodedJwt,
      refreshToken
    };
    (0, _helpers.saveForwardedSession)(nextJsResponse, fronteggSession);
    return fronteggSession;
  } catch (e) {
    logger.error('[refreshToken] Failed to create session e', e);
    return null;
  }
}
//# sourceMappingURL=index.js.map